#!/usr/local/bin/perl
#
#	archie.cgi --- CGI script for Archiesrv
#
#	Copyright (C) 1996  Hiroo Hayashi
#	
#	Author: Hiroo Hayashi <hayashi@pdcd.ilab.toshiba.co.jp>
#	$Id: archie.cgi,v 1.9.1.1 1996-06-03 01:24:25+09 hayashi Exp $
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;

$::debug = 0;
my $ROOTDIR = '/usr/local/lib/archiesrv';
$::ARCHIE = "$ROOTDIR/bin/archie";

unshift (@INC, $ROOTDIR);
require 'archiesrv.conf';

################################################################
print "Content-type:text/html\n\n"; # print HTTP header
&show_env if $::debug;
if ($ENV{QUERY_STRING}) {
    my %v;
    parse_variables(\%v);
    show_result(\%v);
} else {
    show_menu();
}
exit 0;

################################################################
# parse environment varilable "QUERY_STRING"
sub parse_variables {
    my $v = $_[0];

    for (split(/&/, $ENV{QUERY_STRING})) {
	s/\+/ /;
	my ($var, $val) = split('=');
	
	# Convert %XX from hex numbers to alphanumeric
	$var =~ s/%(..)/pack("c",hex($1))/ge;
	$val =~ s/%(..)/pack("c",hex($1))/ge;
	
	if (defined $v->{$var}) {
	    eval "\$v->{$var} .= ':' . '$val'";
	} else {
	    eval "\$v->{$var} = '$val'";
	}
    }

    if ($::debug) {
	print "<pre>\n";
	foreach (sort keys(%{$v})) {
	    print "$_=$v->{$_}\n";
	}
	print "</pre>\n";
    }
}

################################################################
sub show_result {
    my $v = $_[0];

    # print header
    print "<HTML>\n<HEAD>\n<TITLE>Archie Result</TITLE>\n</HEAD>\n<BODY>\n";
    print "<H3>Results of Archie Search</H3>\n";
    print "<hr>\n"; 

    # make command string for archie
    my %type = ( e   => '-e',
		 S   => '-s',    C => '-c',	R => '-r',
		 's' => '-e -s', c => '-e -c',	r => '-e -r');
    
    my %nice = ('Not Nice At All' => 0,		'Nice'		=> 500,
		'Nicer'		  => 1000,	'Very Nice'	=> 5000,
		'Extremely Nice'  => 10000,	'Nicest'	=> 32765);
    
    (my $hits  = $v->{hits}) =~ s/\D//g;
    (my $query = $v->{query}) =~ s/;`//g;
    my $domain = $v->{domain};

    my $cmd = join(' ', $::ARCHIE,
		   (($v->{order} eq 'date') ? '-t' : ''),
		   "-m '$hits'",
		   "-l $type{$v->{type}}",
		   "-N $nice{$v->{nice}}",
		   "-D '$domain'",
		   "'$query'");
		   
    ################################################################
    # main loop
    print "$cmd<br><br>\n" if $::debug;
    open (ARCHIE, "$cmd |")
	|| die "cannot exec $::ARCHIE : $!\n";
    my $n = 0;
    my $lasthost = my $lastdir = '';
    while (<ARCHIE>) {
	my @info = split(' ');
	my ($host, $mode, $link, $size,
	    $month, $day, $year_hour, $file, $dir) = @info;
	
	print "<menu>\n" unless $n;
	if ($host ne $lasthost) {
	    print "</pre>" if $n;
	    print "<li>  $host<pre>";
	}
	print "$dir\n" if $dir ne $lastdir;
	printf "\t%7d %3s %2d %-5s\t", $size, $month, $day, $year_hour;
	(my $path = "$host$dir/$file") =~ s|/+|/|g;
	print "<a href=\"ftp://$path\">$file</a>\n";
	
	$n++; $lasthost = $host; $lastdir = $dir;
    }
    close(ARCHIE);
    
    ################################################################
    # epilogue
    if ($n) {
	print "</menu>\n";
	print "<hr>Total number of hits are $n<br>\n";
    } else {
	print "No hit for \"$query\"<br>\n";
    }
    print "</BODY>\n</HTML>\n";
}

################################################################
sub show_menu {
    print <<EOM;
<HTML>
<HEAD>
<TITLE>Archie Service</TITLE>
</HEAD>
<BODY>

<H1>Archie Service</H1>

<HR>
<FORM ACTION="$ENV{SCRIPT_NAME}">

Search for:
<INPUT NAME="query"> 
<INPUT TYPE="submit" VALUE="Submit">
<INPUT TYPE="reset" VALUE="Reset">

<P>
Anonymous FTP Servers to Search:
<blockquote>
EOM

    my $site_ref;
    foreach $site_ref (@::site_attr) {
	my $site = $site_ref->{address};
	print('<INPUT TYPE="checkbox" NAME="domain" ',
	      "VALUE=\"$site\"");
	print ' CHECKED' if $site_ref->{checked};
	print ">$site<br>\n";
    }

    print <<'EOM';
</blockquote>

<p>
Search method:
<SELECT NAME="type">
<OPTION VALUE="e"> Exact Match
<OPTION VALUE="S" SELECTED> Case Insensitive Substring Match
<OPTION VALUE="C"> Case Sensitive Substring Match
<OPTION VALUE="R"> Regular Expression Match
<OPTION VALUE="s"> Exact Match else Case Insensitive Substring Match
<OPTION VALUE="c"> Exact Match else Case Sensitive Substring Match
<OPTION VALUE="r"> Exact Match else Regular Expression Match
</SELECT>

<br>
Sort:
<SELECT NAME="order">
<OPTION VALUE="host" SELECTED> by Host
<OPTION VALUE="date" SELECTED> by Date
</SELECT>

<br>
Maximum numbers to search (&lt; 100):
<INPUT NAME="hits" SIZE=3 value="95">

<br>
Nice value (process priority):
<SELECT NAME="nice">
<OPTION>Not Nice At All
<OPTION SELECTED>Nice
<OPTION>Nicer
<OPTION>Very Nice
<OPTION>Extremely Nice
<OPTION>Nicest
</SELECT>

</FORM>

<hr>
<address>Hiroo Hayashi &lt;hayashi\@pdcd.ilab.toshiba.co.jp&gt;</address>
</body>
</html>
EOM
}

################################################################
# print environment variables for debugging
sub show_env {
    print "<pre>\n";
    print "$0 ARGV=@ARGV\n";
    chop(my $PWD = `pwd`);
    print "\$PWD=$PWD\n";
    print "QUERY_STRING=$ENV{QUERY_STRING}\n";
    print "PATH_INFO=$ENV{PATH_INFO}\n";
    print "PATH_TRANSLATED=$ENV{PATH_TRANSLATED}\n";
    print "REQUEST_METHOD=$ENV{REQUEST_METHOD}\n";
    print "CONTENT_LENGTH=$ENV{CONTENT_LENGTH}\n";
    print "SERVER_NAME=$ENV{SERVER_NAME}\n";
    print "SCRIPT_NAME=$ENV{SCRIPT_NAME}\n";
    print "GATEWAY_INTERFACE=$ENV{GATEWAY_INTERFACE}\n";
    print "</pre>\n";
}
